------------------------------------------------------------------------
-- PROJECT      :   MONSOON-TORRENT
-- BOARD        :   LOCAL CONTROL BOARD
-- DEVICE ID    :   VIRTEX 5LX
-- PARENT MODULE:   SYSTEM TOP LEVEL  
-- MODULE       :   8-Bit UART 
-- VERSION      :   1.00 
------------------------------------------------------------------------
-- FILE NAME    :   UART_8.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   4/24/2009
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 10.1.03
------------------------------------------------------------------------
-- DESCRIPTION  : 
-- Adapted from work done by  Ovidiu Lupas on OpenCores.org (see below)
-- Note that the original source name has ben modified to better fit the Torrent firmware paradigm
--===========================================================================--
--  S Y N T H E Z I A B L E    miniUART   C O R E
--  www.OpenCores.Org - January 2000
--  This core adheres to the GNU public license  
-- Design units   : miniUART core for the OCRP-1
-- File name      : miniuart.vhd
-- Purpose        : Implements an miniUART device for communication purposes 
--                  between the OR1K processor and the Host computer through
--                  an RS-232 communication protocol.
--                  
-- Library        : uart_lib.vhd
-- Dependencies   : IEEE.Std_Logic_1164
-- Simulator      : ModelSim PE/PLUS version 4.7b on a Windows95 PC
--===========================================================================--
-------------------------------------------------------------------------------
-- Revision list
-- Version   Author                 Date           Changes
--
-- 0.1      Ovidiu Lupas     15 January 2000       New model
-- 1.0      Ovidiu Lupas     January  2000         Synthesis optimizations
-- 2.0      Ovidiu Lupas     April    2000         Bugs removed - RSBusCtrl
--          the RSBusCtrl did not process all possible situations
--
--        olupas@opencores.org
-------------------------------------------------------------------------------
-- Description    : The memory consists of a dual-port memory addressed by
--                  two counters (RdCnt & WrCnt). The third counter (StatCnt)
--                  sets the status signals and keeps a track of the data flow.
-------------------------------------------------------------------------------
-- Entity for miniUART Unit - 9600 baudrate                                  --
-------------------------------------------------------------------------------
library ieee;
   use ieee.std_logic_1164.all;
   use ieee.numeric_std.all;
library work;
   use work.UART_Def.all;

entity Uart_8 is
  port (
     SysClk_i : in  Std_Logic;  -- System Clock
     Reset_i  : in  Std_Logic;  -- Reset input
     RdRx_i   : in  Std_Logic;  -- Read Rx data, cancel RxRdy_o flag
     WrTx_i   : in  Std_Logic;  -- Write data to Tx buffer
     RxD_i    : in  Std_Logic;  -- Serial input
     TxD_o    : out Std_Logic;  -- Serial output
     RxRdy_o  : out Std_Logic;  -- Receive data is valid
     TxRdy_o  : out Std_Logic;  -- Transmiter is ready for a new byte
     Ferror_o : out Std_Logic;  -- RX receive framing error
     Oerror_o : out Std_logic;  -- RX Overrun error
     Data_i   : in  Std_Logic_Vector(7 downto 0);
     Data_o   : out Std_Logic_Vector(7 downto 0));
end entity; --================== End of entity ==============================--
-------------------------------------------------------------------------------
-- Architecture for miniUART Controller Unit
-------------------------------------------------------------------------------
architecture behavour of Uart_8 is
  -----------------------------------------------------------------------------
  -- Signals
  -----------------------------------------------------------------------------
  signal EnabRx : Std_Logic;  -- Enable RX unit
  signal EnabTx : Std_Logic;  -- Enable TX unit
  -----------------------------------------------------------------------------
  -- Baud rate Generator
  -----------------------------------------------------------------------------
  component ClkUnit is
   port (
     SysClk   : in  Std_Logic;  -- System Clock
     EnableRX : out Std_Logic;  -- Control signal
     EnableTX : out Std_Logic;  -- Control signal
     Reset    : in  Std_Logic); -- Reset input
  end component;
  -----------------------------------------------------------------------------
  -- Receive Unit
  -----------------------------------------------------------------------------
  component RxUnit is
  port (
     Clk    : in  Std_Logic;  -- Clock signal
     Reset  : in  Std_Logic;  -- Reset input
     Enable : in  Std_Logic;  -- Enable input
     RxD    : in  Std_Logic;  -- RS-232 data input
     RD     : in  Std_Logic;  -- Read data signal
     FErr   : out Std_Logic;  -- Status signal
     OErr   : out Std_Logic;  -- Status signal
     DRdy   : out Std_Logic;  -- Status signal
     DataIn : out Std_Logic_Vector(7 downto 0));
  end component;
  -----------------------------------------------------------------------------
  -- Transmitter Unit
  -----------------------------------------------------------------------------
  component TxUnit is
  port (
     Clk    : in  Std_Logic;  -- Clock signal
     Reset  : in  Std_Logic;  -- Reset input
     Enable : in  Std_Logic;  -- Enable input
     Load   : in  Std_Logic;  -- Load transmit data
     TxD    : out Std_Logic;  -- RS-232 data output
     TBufE  : out Std_Logic;  -- Tx buffer empty
     DataO  : in  Std_Logic_Vector(7 downto 0));
  end component;
begin
  -----------------------------------------------------------------------------
  -- Instantiation of internal components
  -----------------------------------------------------------------------------
  ClkDiv  : ClkUnit port map (SysClk_i,EnabRX,EnabTX,Reset_i); 
  TxDev   : TxUnit port map (SysClk_i,Reset_i,EnabTX,WrTx_i,TxD_o,TxRdy_o,Data_i);
  RxDev   : RxUnit port map (SysClk_i,Reset_i,EnabRX,RxD_i,RdRx_i,Ferror_o,Oerror_o,RxRdy_o,Data_o);
  
 end behavour;
